( function( $ ) {

  var paymentSubmit = false;

  var signaturePad;

  /**
   * Invoice Functionality
   *
   * @type {Object}
   */
  var rweInvoice = {

    /**
     * Accept the invoice terms of agreement.
     */
    acceptTerms: function( e ) {

      e.preventDefault();

      $( '.rwe-terms-form' ).find( '.check-input, .js-accept-terms' ).attr( 'disabled', 'disabled' );

      $( '.rwe-terms-form' ).children().each( function() {

        $( this ).fadeTo( 'fast', 0.5 );

      } );

      $( '.rwe-terms-form' ).append( rwElephant.preloader );

      // 0: E-Signature
      // 1: Uploaded Signature (custom image)
      // 2: Typed Name
      var signatureType = $( 'ul.verif-menu li.active' ).index(),
          signatureImg;

      if ( 0 === signatureType ) {

        signatureImg  = signaturePad.toDataURL();

      }

      if ( 2 === signatureType ) {

        signatureImg  = '';

      }

      var data = {
        action:   'accept_terms',
        invoiceID: rwElephant.invoiceID, // Note: Do not remove. Required for AJAX  calls.
        termName: $( '.rwe-terms-form' ).find( '#typename' ).val(),
        termEmail: $( '.rwe-terms-form' ).find( '#email' ).val(),
        signaturePNG: signatureImg,
        _wpnonce: rwElephant._nonce_terms
      };

      if ( 1 !== signatureType ) {

        $.post( rwElephant.ajaxurl, data, function( response ) {

          if ( ! response.success ) {

            $( '.rwe-terms-form' ).find( '.check-input, .js-accept-terms' ).removeAttr( 'disabled' );

            $( '.rwe-terms-form' ).children().each( function() {
              $( this ).fadeTo( 'fast', 1 );
            } );

            $( '.rwe-terms-form' ).find( '.rwe-circle-preloader' ).remove();

            console.error( response.data );

            return;

          }

          $( '.results' ).replaceWith( response.data );

        } );

      }

      // Custom uploaded signature image
      if ( 1 === signatureType ) {

        var reader = new FileReader();

        reader.readAsDataURL( document.getElementById( 'sigupload' ).files[0] );

        reader.onload = function () {

          data.signaturePNG = reader.result;

          $.post( rwElephant.ajaxurl, data, function( response ) {

            if ( ! response.success ) {

              $( '.rwe-terms-form' ).find( '.check-input, .js-accept-terms' ).removeAttr( 'disabled' );

              $( '.rwe-terms-form' ).children().each( function() {
                $( this ).fadeTo( 'fast', 1 );
              } );

              $( '.rwe-terms-form' ).find( '.rwe-circle-preloader' ).remove();

              console.error( response.data );

              return;

            }

            $( '.results' ).replaceWith( response.data );

          } );
        };
        reader.onerror = function ( error ) {

          console.error( 'FileReader Error: ' + error );

        };

      }

    },

    /**
     * Submit an invoice payment.
     */
    submitPayment: function( e ) {

      e.preventDefault();

      var paymentAmount = rweInvoice.getPaymentAmount();

      // Invalid payment amount.
      if ( false === paymentAmount ) {

        alert( rwElephant.stripe.invalidAmount );

        return;

      }

      $( '.rwe-payment-form' ).find( 'input' ).attr( 'disabled', 'disabled' );

      $( '.rwe-payment-form' ).children().each( function() {

        $( this ).fadeTo( 'fast', 0.5 );

      } );

      $( '.rwe-payment-form' ).append( rwElephant.preloader );

      StripeCheckout.open( {
        key: rwElephant.stripe.publishableKey,
        amount: parseInt( paymentAmount * 100 ),
        name: rwElephant.stripe.companyName,
        billingAddress: rwElephant.stripe.requireBilling,
        panelLabel: rwElephant.stripe.btnText,
        image: rwElephant.stripe.companyLogo,
        token: function( response ) {

          paymentSubmit = true;

          /* Post payment to RW Elephant servers */
          var data = {
            invoiceID: rwElephant.invoiceID, // Note: Do not remove. Required for AJAX  calls.
            action:   'submit_payment',
            stripeKey: response.id,
            stripeEmail: response.email,
            paymentAmount: parseFloat( paymentAmount ),
            cardForBalance: $( '.js-due-balance' ).is( ':checked' ),
            _wpnonce: rwElephant._nonce_pay
          };

          $.post( rwElephant.ajaxurl, data, function( response ) {

            if ( ! response.success ) {

              $( '.rwe-order-payment__payment-notice' ).remove();

              rweInvoice.displayNotice( 'error', response.data );

              return;

            }

            $( '.rwe-order-payment' ).html( response.data );

          } );

        },
        closed: function() {

          if ( paymentSubmit ) {

            return;

          }

          rweInvoice.reEnableForm();

        }
      } );

    },

    /**
     * Re-enable the form after it has been locked.
     */
    reEnableForm: function() {

      $( '.rwe-payment-form' ).find( 'input' ).removeAttr( 'disabled' );

      $( '.rwe-payment-form' ).children().each( function() {

        $( this ).fadeTo( 'fast', 1 );

      } );

      $( '.rwe-payment-form' ).find( '.rwe-circle-preloader' ).remove();

    },

    displayNotice: function( type, error ) {

      $( '.rwe-order-payment' ).prepend( '<div class="rwe-order-payment__payment-notice"><div class="rwe-payment-notice"><p class="rwe-payment-notice__text ' + type + '">' + error + '</p></div></div>' );

    },

    /**
     * Get the amount of the invoice payment.
     */
    getPaymentAmount: function() {

      var paymentAmount = $( 'input[name=payment]:checked' ).val();

      if ( 'other' === paymentAmount ) {

        paymentAmount = $( '#setAmount' ).val().trim();

      }

      // Empty value, return false and do not process
      if ( '' === paymentAmount ) {

        return false;

      }

      return paymentAmount;

    },

    /**
     * When user focuses on 'Other Amount', check the radio.
     */
    checkCustomAmount: function() {

      $( '#otherAmount' ).click();
      $( '#setAmount' ).attr( 'required', 'required' );

    },

    /**
     * When user clicks on something other than 'Other Amount',
     * remove required attr on custom field
     */
    removeCustomAmountRequired: function() {

      $( '#setAmount' ).removeAttr( 'required' );

    },

  };

  var rweInvoiceSignature = {

    init: function() {

      if ( ! $( '#termsSig' ).length ) {

        return;

      }

      // Signature Canvas (Using Signature Pad - https://github.com/szimek/signature_pad)
      var wrapper     = document.getElementById( 'termsSig' ),
          clearButton = wrapper.querySelector( '[data-action=clear]' ),
          saveButton  = $( 'input[data-action="save_signature"]' ),
          canvas      = document.getElementById( 'signature-pad' );

      signaturePad = new SignaturePad( canvas, {
        maxWidth: '2',
        backgroundColor: 'rgb(255, 255, 255)'
      } );

      function resizeCanvas() {
        var ratio =  Math.max(window.devicePixelRatio || 1, 1);

        canvas.width  = canvas.offsetWidth * ratio;
        canvas.height = canvas.offsetHeight * ratio;

        canvas.getContext('2d').scale(ratio, ratio);

        signaturePad.clear();
      }

      // Store the window width & orientation
      var windowWidth = window.innerWidth;

      // Resize Event
      window.addEventListener('resize', function() {

        // Check if window width has really changed and it's not just
        // iOS triggering a resize event on scroll
        if (window.innerWidth != windowWidth) {

          // Update window width
          windowWidth = window.innerWidth;

          // Resize the Canvas
          resizeCanvas();

        }

      } );

      resizeCanvas();

      clearButton.addEventListener( 'click', function ( event ) {
        signaturePad.clear();
      } );

    },

    tabbedVerificationMethods: function() {

      // Tabbed Form Menu
      $( '.verif-content').slice(1).hide();
      $( '.verif-menu li').eq(0).addClass('active');

      $( '.verif-menu li a').click( function(e) {
        e.preventDefault();
        var content = $(this).attr('href');
        $(this).parent().addClass('active');
        $(this).parent().siblings().removeClass('active');
        $(content).show();
        $(content).siblings('.verif-content').hide();
      } );

    },

  };

  // Invoice
  $( 'body' ).on( 'submit', '.rwe-terms-form', rweInvoice.acceptTerms );
  $( 'body' ).on( 'submit', '.rwe-payment-form', rweInvoice.submitPayment );
  $( 'body' ).on( 'focus', '#setAmount', rweInvoice.checkCustomAmount );
  $( 'body' ).on( 'click', '#otherAmount', function() {
    $( '#setAmount' ).attr( 'required', 'required' );
  } );
  $( 'body' ).on( 'click', '#fullAmount, #minAmount', rweInvoice.removeCustomAmountRequired );

  // Signature
  $( 'body' ).ready( rweInvoiceSignature.init );
  $( 'body' ).ready( rweInvoiceSignature.tabbedVerificationMethods );

} )( jQuery );
